#ifndef Lab_Cloud_h
#define Lab_Cloud_h

// Inheritance :
#include <Lab/Core/BaseObject.h>

#include <AIDA/ICloud1D.h>
#include <AIDA/ICloud2D.h>
#include <AIDA/ICloud3D.h>

namespace AIDA {
  class IHistogram;
}

// Compared to std::vector, BigVector gains 
// at fill time, but wastes time when clearing (then deleting).

#define LAB_USE_BIG_VECTOR

#ifdef LAB_USE_BIG_VECTOR
#include <Lib/BigVector.h>
#endif

#include <Lab/Core/Annotation.h>

namespace Lab {

class Histogram1D;
class Histogram2D;
class Histogram3D;

class BaseCloud : public BaseObject {
public:
protected:
  BaseCloud(IPrinter&,const std::string&,int);
  BaseCloud(IPrinter&,const std::string&,const BaseCloud&);
public:
  virtual ~BaseCloud();
  bool isValid() const;
private:
  virtual void clear() = 0;
protected:
  std::string fTitle;
  double fSw;
  int fLimit;
  Annotation fAnnotation;
#ifdef LAB_USE_BIG_VECTOR
  Lib::BigVector<double> fWs;
#else
  std::vector<double> fWs;
#endif
};

class Cloud1D 
:public BaseCloud
,public virtual AIDA::ICloud1D {
public: //AIDA/IBaseHistogram
  virtual void* cast(const std::string&) const;
  virtual std::string title() const;
  virtual bool setTitle(const std::string&);
  virtual AIDA::IAnnotation& annotation();
  virtual const AIDA::IAnnotation& annotation() const;
  virtual int dimension() const;
  virtual bool reset();
  virtual int entries() const;
public: //AIDA/ICloud
  virtual double sumOfWeights() const;
  virtual bool convertToHistogram();
  virtual bool isConverted() const;
  virtual int maxEntries() const;
  virtual bool scale(double);
public: //AIDA/ICloud1D
  virtual bool fill(double,double = 1);
  virtual double lowerEdge() const;
  virtual double upperEdge() const;
  virtual double value(int) const;
  virtual double weight(int) const;
  virtual double mean() const;
  virtual double rms() const;
  virtual bool convert(int,double,double);
  virtual bool convert(const std::vector<double>&);
  virtual const AIDA::IHistogram1D& histogram() const;
  virtual bool fillHistogram(AIDA::IHistogram1D&) const;
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public: //Lab::BaseCloud
  virtual void clear();
  bool setHistogram(Histogram1D*);
public:
  Cloud1D(IPrinter&,
          const std::string&,const std::string&,int=-1);
  Cloud1D(const std::string&,const Cloud1D&);
  virtual ~Cloud1D();
private:
#ifdef LAB_USE_BIG_VECTOR
  Lib::BigVector<double> fXs;
#else
  std::vector<double> fXs;
#endif
  double fLowerX;
  double fUpperX;
  double fSxw;
  double fSx2w;
  Histogram1D* fHistogram;
};

class Cloud2D 
:public BaseCloud
,public virtual AIDA::ICloud2D {
public: //AIDA/IBaseHistogram
  virtual void* cast(const std::string&) const;
  virtual std::string title() const;
  virtual bool setTitle(const std::string&);
  virtual AIDA::IAnnotation& annotation();
  virtual const AIDA::IAnnotation& annotation() const;
  virtual int dimension() const;
  virtual bool reset();
  virtual int entries() const;
public: //AIDA/ICloud
  virtual double sumOfWeights() const;
  virtual bool convertToHistogram();
  virtual bool isConverted() const;
  virtual int maxEntries() const;
  virtual bool scale(double);
public: //ICloud2D
  virtual bool fill(double,double,double = 1);
  virtual double lowerEdgeX() const;
  virtual double upperEdgeX() const;
  virtual double lowerEdgeY() const;
  virtual double upperEdgeY() const;
  virtual double valueX(int) const;
  virtual double valueY(int) const;
  virtual double weight(int) const;
  virtual double meanX() const;
  virtual double meanY() const;
  virtual double rmsX() const;
  virtual double rmsY() const;
  virtual bool convert(int,double,double,int,double,double);
  virtual bool convert(const std::vector<double>&,const std::vector<double>&);
  virtual const AIDA::IHistogram2D& histogram() const;
  virtual bool fillHistogram(AIDA::IHistogram2D&) const;
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public: //Lab::BaseCloud
  virtual void clear();
  bool setHistogram(Histogram2D*);
public:
  Cloud2D(IPrinter&,
          const std::string&,const std::string&,int=-1);
  Cloud2D(const std::string&,const Cloud2D&);
  virtual ~Cloud2D();
private:
#ifdef LAB_USE_BIG_VECTOR
  Lib::BigVector<double> fXs;
  Lib::BigVector<double> fYs;
#else
  std::vector<double> fXs;
  std::vector<double> fYs;
#endif
  double fLowerX;
  double fUpperX;
  double fLowerY;
  double fUpperY;
  double fSxw;
  double fSx2w;
  double fSyw;
  double fSy2w;
  Histogram2D* fHistogram;
};

class Cloud3D
:public BaseCloud
,public virtual AIDA::ICloud3D {
public: //AIDA/IBaseHistogram
  virtual void* cast(const std::string&) const;
  virtual std::string title() const;
  virtual bool setTitle(const std::string&);
  virtual AIDA::IAnnotation& annotation();
  virtual const AIDA::IAnnotation& annotation() const;
  virtual int dimension() const;
  virtual bool reset();
  virtual int entries() const;
public: //AIDA/ICloud
  virtual double sumOfWeights() const;
  virtual bool convertToHistogram();
  virtual bool isConverted() const;
  virtual int maxEntries() const;
  virtual bool scale(double);
public: //AIDA/ICloud3D
  virtual bool fill(double,double,double,double = 1);
  virtual double lowerEdgeX() const;
  virtual double upperEdgeX() const;
  virtual double lowerEdgeY() const;
  virtual double upperEdgeY() const;
  virtual double lowerEdgeZ() const;
  virtual double upperEdgeZ() const;
  virtual double valueX(int) const;
  virtual double valueY(int) const;
  virtual double valueZ(int) const;
  virtual double weight(int) const;
  virtual double meanX() const;
  virtual double meanY() const;
  virtual double meanZ() const;
  virtual double rmsX() const;
  virtual double rmsY() const;
  virtual double rmsZ() const;
  virtual bool convert(int,double,double,int,double,double,int,double,double);
  virtual bool convert(const std::vector<double>&,
                       const std::vector<double>&,
                       const std::vector<double>&);
  virtual const AIDA::IHistogram3D& histogram() const;
  virtual bool fillHistogram(AIDA::IHistogram3D&) const;
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public: //Lab::BaseCloud
  virtual void clear();
  bool setHistogram(Histogram3D*);
public:
  Cloud3D(IPrinter&,
          const std::string&,const std::string&,int=-1);
  Cloud3D(const std::string&,const Cloud3D&);
  virtual ~Cloud3D();
private:
#ifdef LAB_USE_BIG_VECTOR
  Lib::BigVector<double> fXs;
  Lib::BigVector<double> fYs;
  Lib::BigVector<double> fZs;
#else
  std::vector<double> fXs;
  std::vector<double> fYs;
  std::vector<double> fZs;
#endif
  double fLowerX;
  double fUpperX;
  double fLowerY;
  double fUpperY;
  double fLowerZ;
  double fUpperZ;
  double fSxw;
  double fSx2w;
  double fSyw;
  double fSy2w;
  double fSzw;
  double fSz2w;
  Histogram3D* fHistogram;
};

}

#endif
