#ifndef Lib_Functions_h
#define Lib_Functions_h

#include <complex>
#include <vector>
#include <string>

class IPrinter;

#include <Lib/Processor.h>

namespace Lib {

class Gaussian {
public:
  Gaussian(double = 1,double = 0,double = 1);
  Gaussian(const Gaussian&);
  double value(double) const;
  void set(double,double,double);
protected:
  double fScale,fMean,fSigma;
};

class BreitWigner {
public:
  BreitWigner(double = 1,double = 0,double = 1);
  BreitWigner(const BreitWigner&);
  double value(double) const;
  void set(double,double,double);
protected:
  double fHeight,fCenter,fWidth;
};

class Exponential {
public:
  Exponential(double = 0,double = 1);
  Exponential(const Exponential&);
  double value(double) const;
  void set(double,double);
protected:
  double fA,fB;
};

class Polynomial {
public:
  Polynomial(const std::vector<double>&);
  Polynomial(const Polynomial&);
  double value(double) const;
  void set(const std::vector<double>&);
protected:
  std::vector<double> fPs;
};

class ArrayFunction {
public:
  ArrayFunction();
  ArrayFunction(const ArrayFunction&);
  ArrayFunction(const std::vector<double>&,
                const std::vector<double>&);
  double value(double) const;
  void set(const std::vector<double>&,const std::vector<double>&);
  const std::vector<double>& getXs() const;
  const std::vector<double>& getVs() const;
private:
  double interpolate(int,double) const;
private:
  std::vector<double> fXs;
  std::vector<double> fVs;
  int fIndex;
};

class Array2DFunction {
public:
  Array2DFunction();
  Array2DFunction(const Array2DFunction&);
  Array2DFunction(const std::vector<double>&,
                  const std::vector<double>&,
                  const std::vector<double>&);
  double value(double,double) const;
  void set(const std::vector<double>&,
           const std::vector<double>&,
           const std::vector<double>&);
  const std::vector<double>& getXs() const;
  const std::vector<double>& getYs() const;
  const std::vector<double>& getVs() const;
private:
  double interpolate(int,int,double,double) const;
  int offset(int,int) const;
private:
  std::vector<double> fXs;
  std::vector<double> fYs;
  std::vector<double> fVs;
  int fI;
  int fJ;
};

class ComplexArrayFunction {
public:
  ComplexArrayFunction();
  ComplexArrayFunction(const ComplexArrayFunction&);
  ComplexArrayFunction(const std::vector<double>&,
                       const std::vector< std::complex<double> >&);
  ComplexArrayFunction(const std::vector<double>&,
                       const std::vector<double>&,
                       const std::vector<double>&);
  double value(double) const;
  const std::vector<double>& getXs() const;
  const std::vector< std::complex<double> >& getVs() const;
  void set(const std::vector<double>&,
           const std::vector< std::complex<double> >&);
  void set(const std::vector<double>&,
           const std::vector<double>&,
           const std::vector<double>&);
private:
  double interpolate(int,double) const;
private:
  std::vector<double> fXs;
  std::vector< std::complex<double> > fVs;
  int fIndex;
};

class ScriptedFunction {
public:
  ScriptedFunction(const std::string&,IPrinter&);
  ScriptedFunction(const ScriptedFunction&);
  virtual ~ScriptedFunction();
  bool isValid() const;
  double value(double) const;
  void set(const std::string&);
  const std::string& getScript();
  void setVariables(const std::vector<double>&);
  unsigned int getNumberOfVariables();
  double getVariableValue(unsigned int);
  int variableIndex(const std::string&) const;
  const std::vector<std::string>& variableNames() const;
private:
  std::string fScript;
  std::vector<std::string> fVars;
  std::vector<double> fVals;
  IPrinter& fPrinter;
  Processor fProcessor;
};

class ScriptedFunction2D {
public:
  ScriptedFunction2D(const std::string&,IPrinter&);
  ScriptedFunction2D(const ScriptedFunction2D&);
  virtual ~ScriptedFunction2D();
  bool isValid() const;
  double value(double,double) const;
  void set(const std::string&);
  const std::string& getScript();
  void setVariables(const std::vector<double>&);
  unsigned int getNumberOfVariables();
  double getVariableValue(unsigned int);
  int variableIndex(const std::string&) const;
  const std::vector<std::string>& variableNames() const;
private:
  std::string fScript;
  std::vector<std::string> fVars;
  std::vector<double> fVals;
  IPrinter& fPrinter;
  Processor fProcessor;
};

}

#endif




