#ifndef Lib_ISession_h
#define Lib_ISession_h

/**
 * Pure abstract interface for a session.
 *
 * A session manages named manager.
 * 
 * @author G.Barrand
 */

#include <string>
#include <vector>

class IManager;
class IPrinter;

class ISession {
public:
  virtual ~ISession() {};
public:
  /**
   * Add a manager.
   */
  virtual void addManager(IManager*) = 0;

  /**
   * Remove a manager.
   */
  virtual void removeManager(IManager*) = 0;

  /**
   * Find a manager knowing its name.
   * @param aName Name of the manager to find.
   * @return The manager, zero is not found.
   */
  virtual IManager* findManager(const std::string& aName) const = 0;

  /**
   * Destroy a manager.
   * @param aName Name of the manager to destroy.
   */
  virtual void destroyManager(const std::string& aName) = 0;

  /**
   * Return the printer. A session has always a printer.
   * @return The printer.
   */
  virtual IPrinter& printer() const = 0;

  /**
   * Load a DLL. If the aPackage argument is given, an "extern C"
   * function "aPackageInitialize(ISession&)" is searched and executed.
   * @param aFile The DLL to load.
   * @param aPackage Package name.
   * @return The status of the execution.
   */
  virtual bool load(const std::string& aFile,const std::string& aPackage) = 0;

  /**
   * Permit to redirect or not the printer output to stdout.
   * @param aRedirect If false, output are redirected to stdout.
   */
  virtual void redirectOutput(bool aRedirect) = 0;

  /**
   * A session manages a list of string keyed parameters.
   * It permits to set a parameter.
   * @param aKey The string key.
   * @param aValue The value.
   */
  virtual bool setParameter(const std::string& aKey,const std::string& aValue) = 0;

  /**
   * Retreive the value of a parameter knowing its key.
   * @param aKey The string key.
   * @param aValue The value.
   * @return The status of the execution.
   */
  virtual bool parameterValue(const std::string& aKey,std::string& aValue) = 0;

  /**
   * Get list of the available parameters (implementation-dependent)
   */
  virtual std::vector<std::string> availableParameters() const = 0;

  /**
   * Flush the printer output.
   */
  virtual void flush() = 0;

  /**
   * Set the verbosity of the session.
   * @param aVerbosity The verbosity. If zero not verbose.
   */
  virtual void setVerboseLevel(int aVerbosity) = 0;

  /**
   * @return The verbosity of the session.
   */
  virtual int verboseLevel() const = 0;

  /**
   * Find manager of a given class. The IManager->cast(aClass) 
   * is used to test the class of a manager.
   * @param aClass Class to search. if aClass=="" all managers are returned.
   * @return The list of found managers.
   */
  virtual std::vector<std::string> managerNames(const std::string& aClass) const = 0;
};

#endif
