#ifndef Lib_Matrix_h
#define Lib_Matrix_h

#include <vector>

namespace Lib {

template <class T>
class Matrix {
 public:
  enum {NotFound = -1};
  Matrix(int aOrder = 1):fOrder(aOrder) {
    if(aOrder<1) fOrder = 1;
    int n = fOrder * fOrder;
    fMatrix.resize(n);
    for(int index=0;index<n;index++) fMatrix[index] = 0;
  }
  Matrix(int aOrder,const std::vector<T>& aValues):fOrder(aOrder) {
    if(aOrder<1) fOrder = 1;
    int n = fOrder * fOrder;
    fMatrix.resize(n);
    if(set(aValues)) return;
    for(int index=0;index<n;index++) fMatrix[index] = 0;
  }
  Matrix(const Matrix& aMatrix) {
    fOrder = aMatrix.fOrder;
    int n = fOrder * fOrder;
    fMatrix.resize(n);
    for(int index=0;index<n;index++) fMatrix[index] = aMatrix.fMatrix[index];
  }
  virtual ~Matrix() {}
  int order() const { return fOrder;}
  T value(int aRow,int aCol) const {
    int index = offset(aRow,aCol);
    if(index==NotFound) {
      // FIXME : print an error message.
      return 0;
    } 
    return fMatrix[index];
  }
  T value(int aIndex) const {
    // Not protected against under,overflow.
    return fMatrix[aIndex];
  }
  void mul(const T& aT) {
    int n = fOrder * fOrder;
    for(int index=0;index<n;index++) fMatrix[index] *= aT;
  }
  void mul(const Matrix& aMatrix) {
    if(aMatrix.fOrder!=fOrder) return;
    Matrix tmp(*this);
    for(int c=0;c<fOrder;c++) {
      for(int r=0;r<fOrder;r++) {
        T value = 0;
        for(int i=0;i<fOrder;i++) {
          value += tmp.fMatrix[offset(r,i)] * aMatrix.fMatrix[offset(i,c)];
        }
        fMatrix[offset(r,c)] = value;
      }
    }
  }
  void sub(const Matrix& aMatrix) {
    if(aMatrix.fOrder!=fOrder) return;
    for(int r=0;r<fOrder;r++) {
      for(int c=0;c<fOrder;c++) {
        int index = offset(r,c);
        fMatrix[index] = fMatrix[index] - aMatrix.fMatrix[index];
      }
    }
  }
  void add(const Matrix& aMatrix) {
    if(aMatrix.fOrder!=fOrder) return;
    for(int r=0;r<fOrder;r++) {
      for(int c=0;c<fOrder;c++) {
        int index = offset(r,c);
        fMatrix[index] = fMatrix[index] + aMatrix.fMatrix[index];
      }
    }
  }
  bool set(const std::vector<T>& aValues) {
    int n = fOrder * fOrder;
    if((int)aValues.size()!=n) return false;
    for(int index=0;index<n;index++) fMatrix[index] = aValues[index];
    return true;
  }
  Matrix& operator=(const Matrix& aMatrix) {
    if(this==&aMatrix) return *this; // Same matrix.
    fMatrix.clear();
    fOrder = aMatrix.fOrder;
    int n = fOrder * fOrder;
    fMatrix.resize(n);
    for(int index=0;index<n;index++) fMatrix[index] = aMatrix.fMatrix[index];
    return *this;
  }
  bool equal(const Lib::Matrix<T>& aMatrix) const {
    if(fOrder!=aMatrix.fOrder) return false;
    int n = fOrder * fOrder;
    for(int index=0;index<n;index++) {
      if(fMatrix[index]!=aMatrix.fMatrix[index]) return false;
    }
    return true;
  }
  T trace() const {
    T value = 0;
    for(int index=0;index<fOrder;index++) 
      value += fMatrix[offset(index,index)];
    return value;
  }
 protected:
  int offset(int aRow,int aCol) const {
    if((aRow<0) || (aRow>=fOrder)) return NotFound;
    if((aCol<0) || (aCol>=fOrder)) return NotFound;
    return fOrder * aRow + aCol;
  }
 protected:
  int fOrder;
  std::vector<T> fMatrix;
};

}

template <class T>
Lib::Matrix<T> operator*(const Lib::Matrix<T>& a1,const Lib::Matrix<T>& a2) {
  Lib::Matrix<T> tmp = a1;
  tmp.mul(a2);
  return tmp;
}

template <class T>
Lib::Matrix<T> operator*(const T& a1,const Lib::Matrix<T>& a2) {
  Lib::Matrix<T> tmp = a2;
  tmp.mul(a1);
  return tmp;
}

template <class T>
Lib::Matrix<T> operator+(const Lib::Matrix<T>& a1,const Lib::Matrix<T>& a2) {
  Lib::Matrix<T> tmp = a1;
  tmp.add(a2);
  return tmp;
}

template <class T>
Lib::Matrix<T> operator-(const Lib::Matrix<T>& a1,const Lib::Matrix<T>& a2) {
  Lib::Matrix<T> tmp = a1;
  tmp.sub(a2);
  return tmp;
}

template <class T>
bool operator==(const Lib::Matrix<T>& a1,const Lib::Matrix<T>& a2) {
  return a1.equal(a2);
}

template <class T>
bool operator!=(const Lib::Matrix<T>& a1,const Lib::Matrix<T>& a2) {
  return (a1==a2 ? false:true);
}

#endif
