#ifndef Lib_Vector_h
#define Lib_Vector_h

#include <vector>

namespace Lib {

// Vector handles an hyperparallelepiped of cells of class T.

template <class T>
class Vector {
public:
  enum {NotFound = -1};
  typedef typename std::vector< std::pair<int,int> > Cut;
  Vector() {
  }
  Vector(const std::vector<int>& aOrders) {
    configure(aOrders);
  }
  Vector(int aDimension,int aOrder) { 
    // A hypercube of dimension "aDimension" and size "aOrder".
    int dim = aDimension<0?0:aDimension;
    std::vector<int> orders(dim);
    for(int index=0;index<dim;index++) orders[index] = aOrder;
    configure(orders);
  }
  Vector(const Vector& aFrom)
  :fOrders(aFrom.fOrders)
  ,fOffsets(aFrom.fOffsets)
  ,fVector(aFrom.fVector)
  ,fIs(aFrom.fIs){
  }
  virtual ~Vector() {}
  Vector& operator=(const Vector& aFrom) {
    fOrders = aFrom.fOrders;
    fOffsets = aFrom.fOffsets;
    fVector = aFrom.fVector;
    fIs = aFrom.fIs;
    return *this;
  }
  void copy(const Vector& aFrom) {
    fOrders = aFrom.fOrders;
    fOffsets = aFrom.fOffsets;
    fVector = aFrom.fVector;
    fIs = aFrom.fIs;
  }
  void clear() {
    fOrders.clear();
    fOffsets.clear();
    fVector.clear();
    fIs.clear();
  }
  void configure(const std::vector<int>& aOrders) {
    fOrders = aOrders;
    int dim = fOrders.size();
    if(dim<=0) {
      clear();
      return;
    }
    int size = 1;
    for(int index=0;index<dim;index++) {
      if(fOrders[index]<0) {
        clear();
        return;
      }
      size *= fOrders[index];
    }
    fVector.resize(size,0);
    fOffsets.resize(dim,0);
    fOffsets[0] = 1;
    for(int iaxis=1;iaxis<dim;iaxis++) 
      fOffsets[iaxis] = fOffsets[iaxis-1] * fOrders[iaxis-1];
    fIs.resize(dim,0);
  }
  int dimension() const { return fOrders.size();}
  const std::vector<int>& orders() const { return fOrders;}
  int size() const {return fVector.size();}
  bool setValue(const std::vector<int>& aIs,const T& aValue) {
    int off = offset(aIs);
    if(off==NotFound) return false;
    fVector[off] = aValue;
    return true;
  }
  bool value(const std::vector<int>& aIs,T& aValue) const {
    int off = offset(aIs);
    if(off==NotFound) {
      aValue = 0;
      return false;
    }
    aValue = fVector[off];
    return true;
  }
  void reset() {
    int size = fVector.size();
    for(int index=0;index<size;index++) fVector[index] = 0;
  }
  const std::vector<T>& vector() const { return fVector;}
  std::vector<T>& vector() { return fVector;}
  bool fill(const std::vector<T>& aValues,Cut* aCut = 0) {
    unsigned int dsize = aValues.size();
    unsigned di = 0;
    unsigned int vsize = fVector.size();
    for(unsigned int index=0;index<vsize;index++) {
      if(!aCut || (aCut && accept(index,*aCut)) ) {
        if(di>=dsize) return false; //aValues exhausted too early
        fVector[index] = aValues[di];
        di++;
      }
    }
    return true;
  }
  // Related to other Vector :
  bool equal(const Vector& aVector) const {
    if(fOrders!=aVector.fOrders) return false;
    unsigned int vsize = fVector.size();
    for(unsigned int index=0;index<vsize;index++) {
      if(fVector[index]!=aVector.fVector[index]) return false;
    }
    return true;
  }
  bool add(const Vector& aVector,Cut* aCut = 0) {
    if(fOrders!=aVector.fOrders) return false;
    unsigned int vsize = fVector.size();
    for(unsigned int index=0;index<vsize;index++) {
      if(!aCut || (aCut && accept(index,*aCut)) ) {
        fVector[index] += aVector.fVector[index];
      }
    }
    return true;
  }
  bool subtract(const Vector& aVector) {
    if(fOrders!=aVector.fOrders) return false;
    unsigned int vsize = fVector.size();
    for(unsigned int index=0;index<vsize;index++) 
      fVector[index] -= aVector.fVector[index];
    return true;
  }
  bool multiply(const Vector& aVector) {
    if(fOrders!=aVector.fOrders) return false;
    unsigned int vsize = fVector.size();
    for(unsigned int index=0;index<vsize;index++) 
      fVector[index] *= aVector.fVector[index];
    return true;
  }
  bool divide(const Vector& aVector) {
    if(fOrders!=aVector.fOrders) return false;
    bool status = true;
    unsigned int vsize = fVector.size();
    for(unsigned int index=0;index<vsize;index++) {
      if(aVector.fVector[index]==0)  {
        fVector[index] = 0; //PAW convention.
        status = false;
      } else {
        fVector[index] /= aVector.fVector[index];
      }
    }
    return status;
  }
  void add(const T& aT,Cut* aCut = 0) {
    unsigned int vsize = fVector.size();
    for(unsigned int index=0;index<vsize;index++) {
      if(!aCut || (aCut && accept(index,*aCut)) ) {
        fVector[index] += aT;
      }
    }
  }
  void multiply(const T& aT) {
    unsigned int vsize = fVector.size();
    for(unsigned int index=0;index<vsize;index++) 
      fVector[index] *= aT;
  }
  bool invert() {
    unsigned int vsize = fVector.size();
    bool status = true;
    for(unsigned int index=0;index<vsize;index++) {
      if(fVector[index]==0)  {
        fVector[index] = 0; //PAW convention.
        status = false;
      } else {
        fVector[index] = 1./fVector[index];
      }
    }
    return status;
  }
  int offset(const std::vector<int>& aIs) const {
    unsigned int dim = fOrders.size();
    if(aIs.size()!=dim) return NotFound;
    if(dim<=0) return NotFound;
    int value = 0;
    for(unsigned int iaxis=0;iaxis<dim;iaxis++) { 
      int i = aIs[iaxis];
      if((i<0)||(i>=fOrders[iaxis])) return NotFound;
      value += i * fOffsets[iaxis];
    }
    return value;
  }
  bool indices(int aOffset,std::vector<int>& aIs) const {
    if((aOffset<0)||(aOffset>=(int)fVector.size())) return false;
    unsigned int dim = fOrders.size();
    int off = aOffset;
    for(int iaxis=dim-1;iaxis>=0;iaxis--) { 
      aIs[iaxis] = off/fOffsets[iaxis];
      off -= aIs[iaxis] * fOffsets[iaxis];
    }
    return true;
  }
  bool accept(int aIndex,const Cut& aCut) const {
    unsigned int dim = fOrders.size();
    if(aCut.size()!=dim) return false;
#define Lib_Vector_self (*((Lib::Vector<T>*)this))
    if(!indices(aIndex,Lib_Vector_self.fIs)) return false;
#undef Lib_Vector_self
    bool good = true;
    for(unsigned iaxis=0;iaxis<dim;iaxis++) { 
      if(fIs[iaxis]<aCut[iaxis].first) {good = false;break;}
      if(fIs[iaxis]>aCut[iaxis].second) {good = false;break;}
    }
    return good;
  }
protected:
  std::vector<int> fOrders;
  std::vector<int> fOffsets;
  std::vector<T> fVector;
  std::vector<int> fIs;
};

}

#endif




