#ifndef OnX_Callbacks_Inventor_h
#define OnX_Callbacks_Inventor_h

//
//  This file contains static helper methods.
// It is intended to be included in OnX Callbacks file
// to ease callbacks programming.
//  It relies on interfaces and basic libraries (Lib, stdcxx) only.
//

// Contain Inventor related helpers.
// Then using this file in callbacks will induce to link
// the callback library to HEPVis and Inventor.

#include <Lib/Interfaces/ISession.h>
#include <Lib/Manager.h>
#include <Lib/fmanip.h>
#include <Lib/Cast.h>

#include <OnX/Interfaces/IUI.h>
#include <OnX/Interfaces/IWidget.h>
#include <OnX/Interfaces/IPageViewer.h>

#include <Inventor/SoDB.h>
#include <Inventor/SoInput.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/actions/SoWriteAction.h>

#include <HEPVis/nodekits/SoRegion.h>
#include <HEPVis/nodekits/SoPage.h>
#include <HEPVis/actions/SoAlternateRepAction.h>

class SoCamera;
class SoLightModel;

//////////////////////////////////////////////////////////////////////////////
/// SoPage ///////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
static SoPage* ui_SoPage(
 IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  IWidget* w = aUI.findWidget(aUI.currentViewer());
  if(!w) return 0;
  IPageViewer* pageViewer = Lib_Cast(*w,IPageViewer);
  if(!pageViewer) return 0;
  return pageViewer->soPage();
}
//////////////////////////////////////////////////////////////////////////////
static bool page_setCurrentRegion(
 SoPage& aSoPage
,int aIndex
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoRegion* soRegion = aSoPage.getRootRegion(aIndex);
  if(!soRegion) return false;
  aSoPage.highlightRegion(soRegion);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
static SoRegion* page_currentRegion(
 SoPage& aPage
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoRegion* highlighted = aPage.getHighlightedRegion();
  if(!highlighted) return 0;
  if(aPage.isRootRegion(highlighted)==FALSE) return 0;
  return highlighted;
}
//////////////////////////////////////////////////////////////////////////////
static void page_connect_current_region(
 SoPage& aPage
,int aIndex
)
//////////////////////////////////////////////////////////////////////////////
// Connect the current region to the "aIndex" region.
// The current region will then display the scene graph of the "aIndex" region.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoRegion* region = page_currentRegion(aPage);
  if(!region) return;
  SoRegion* fromRegion = aPage.getRootRegion(aIndex);
  if(!fromRegion) return;
  region->connectFrom(fromRegion);
}
//////////////////////////////////////////////////////////////////////////////
static SoRegion* page_createRegion(
 SoPage& aPage
,const std::string& aClass
,double aX
,double aY
,double aWidth
,double aHeight                           
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoRegion* soRegion = aPage.createRegion
    (aClass.c_str(),(float)aX,(float)aY,(float)aWidth,(float)aHeight);
  if(aPage.getNumberOfRootRegions()==1) aPage.highlightRegion(soRegion);
  return soRegion;
}
//////////////////////////////////////////////////////////////////////////////
/// SoRegion /////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
static SoRegion* ui_SoRegion(
 IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return 0;
  return page_currentRegion(*soPage);
}
//////////////////////////////////////////////////////////////////////////////
static SoCamera* region_camera(
 SoRegion& aRegion
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (SoCamera*)aRegion.findNode("type","SoCamera");
}
//////////////////////////////////////////////////////////////////////////////
static SoLightModel* region_lightModel(
 SoRegion& aRegion
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (SoLightModel*)aRegion.findNode("type","SoLightModel");
}
//////////////////////////////////////////////////////////////////////////////
static SoSeparator* region_staticNode(
 SoRegion& aRegion
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (SoSeparator*)aRegion.findNode("part","staticScene");
}
//////////////////////////////////////////////////////////////////////////////
static SoSeparator* region_dynamicNode(
 SoRegion& aRegion
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (SoSeparator*)aRegion.findNode("part","dynamicScene");
}
//////////////////////////////////////////////////////////////////////////////
static bool region_readInventorFile(
 SoRegion& aRegion
,const std::string& aFile
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Solve environment variables :
  std::string name = Lib::fmanip::name(aFile);

  SoInput soInput;
  if(!soInput.openFile(name.c_str())) return false;

  SoSeparator* separator = SoDB::readAll(&soInput);
  if(!separator) return false;

  if(aRegion.doIt(SbAddNode(separator,"staticScene"))==FALSE)
    separator->unref();

  return true;
}
//////////////////////////////////////////////////////////////////////////////
static bool region_writeInventorFile(
 SoRegion& aRegion
,const std::string& aFile
,bool aBinary
,const std::string& aWhat
,bool aGenAlternateRep
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoNode* node = aRegion.getWriteNode(aWhat.c_str());
  if(!node) return false;

  SoAlternateRepAction alternateRepAction;
  if(aGenAlternateRep) {
    alternateRepAction.setGenerate(TRUE); //Generate alternate reps.
    alternateRepAction.apply(node);
  }

  SoWriteAction writeAction;
  writeAction.getOutput()->openFile(aFile.c_str());
  writeAction.getOutput()->setBinary(aBinary?TRUE:FALSE);
  writeAction.apply(node);
  writeAction.getOutput()->closeFile();

  if(aGenAlternateRep) {
    alternateRepAction.setGenerate(FALSE); //Clear alternate reps.
    alternateRepAction.apply(node);
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
static bool region_setNode(
 SoRegion& aRegion
,const std::string& aWhat
,const std::string& aSearch
,const std::string& aValue
) 
//////////////////////////////////////////////////////////////////////////////
// aWhat = [part,name,type]
// aSearch
// aString value
// Example : 
//   region_setNode region "type SoCamera height 500 position 0 0 500"
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoNode* node = aRegion.findNode(aWhat.c_str(),aSearch.c_str());
  if(!node) return false;
  return node->set(aValue.c_str())==TRUE?true:false;
}
//////////////////////////////////////////////////////////////////////////////
static void OnX_Inventor_quiet_cxx()
//////////////////////////////////////////////////////////////////////////////
// To quiet g++ warnings about unused macros.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  IUI* ui = 0;
  ui_SoPage(*ui);
  ui_SoRegion(*ui);

  SoPage* soPage = 0;
  page_setCurrentRegion(*soPage,0);
  page_currentRegion(*soPage);
  page_connect_current_region(*soPage,0);
  page_createRegion(*soPage,"",0,0,0,0);

  SoRegion* region = 0;
  region_camera(*region);
  region_lightModel(*region);
  region_dynamicNode(*region);
  region_staticNode(*region);
  region_readInventorFile(*region,"");
  region_setNode(*region,"","","");
  region_writeInventorFile(*region,"",false,"",true);
}

#endif
