#ifndef OnX_IPage_h
#define OnX_IPage_h

/**
 * Pure abstract interface for a document page.
 *
 * A page manages viewing regions.
 * 
 * @author G.Barrand
 */

#include <string>

class IRegion;

class IPage {
public:
  virtual ~IPage() {}
public: 
  /**
   * Create a grid of regions of a given class and set 
   * the current viewing region. If working with Inventor,
   * the current region is the one that receives the 
   * focus of the Inventor viewer controls. In particular
   * the camera of the viewing region can be manipulated
   * with the Inventor thumbwheels.
   * 
   * @param aCols Number of columns in the grid.
   * @param aRows Number of rows in the grid.
   * @param aIndex Index of the region to set as current.
   * @return The status of the execution.
   */
  virtual bool createRegions(int aCols,int aRows,int aIndex,const std::string& aClass = "") = 0;

  /**
   * Create a region of a given class at a given position and 
   * size in normal coordinates of the page. Origin is the
   * lower left cornet of the page. X axis to the right, Y axis
   * to the top.
   * 
   * @param aX x position in [0,1]
   * @param aY y position in [0,1]
   * @param aW width in [0,1]
   * @param aH height in [0,1]
   * @param aClass Class to create.
   * @return The created region.
   */
  virtual IRegion* createRegion(double aX,double aY,
                                double aW,double aH,
                                const std::string& aClass= "") = 0;

  /**
   * Create one region as if in a grid of regions.
   * 
   * @param aCols Number of columns in the grid.
   * @param aRows Number of rows in the grid.
   * @param aIndex Index of the region to set as current.
   * @param aClass Class to create.
   * @return The created region.
   */
  virtual IRegion* createGridRegion(int aCols,int aRows,int aIndex,const std::string& aClass= "") = 0;

  /**
   * Pass from a given grid of regions to a new grid
   * by creating new regions if necessary. Existing regions
   * and their content are preserved if not in excess, else
   * they are deleted.
   * 
   * @param aCols Number of columns in the grid.
   * @param aRows Number of rows in the grid.
   * @param aClass Class to create.
   * @return The status of the execution.
   */
  virtual bool addRegions(int aCols,int aRows,const std::string& aClass = "") = 0;

  /**
   * Set current region (if Inventor, the one that receive the focus
   * of the viewer).
   * 
   * @param aIndex Index of the region to set as current.
   * @return The status of the execution.
   */
  virtual bool setCurrentRegion(int aIndex = 0) = 0;

  /**
   * Set given region as the current.
   * 
   * @param aRegion Region to set as current.
   * @return The status of the execution.
   */
  virtual bool setCurrentRegion(IRegion&) = 0;

  /**
   * Set the next region in the list of managed regions as the current one.
   * 
   * @return The status of the execution.
   */
  virtual bool next() = 0;

  /**
   * Set the previous region in the list of managed regions as the current one.
   * 
   * @return The status of the execution.
   */
  virtual bool previous() = 0;

  /**
   * Set a parameter of the page.
   * 
   * @return The status of the execution.
   */
  virtual bool setParameter(const std::string&,const std::string&) = 0;

  /**
   * Compell a refresh of the managed regions.
   * 
   * @return The status of the execution.
   */
  virtual bool refresh() = 0;

  /**
   * Clear the content of the managed regions.
   * 
   * @return The status of the execution.
   */
  virtual bool clear() = 0;

  /**
   * Delete the current region.
   */
  virtual void deleteRegion() = 0;

  /**
   * Delete all the managed regions.
   */
  virtual void deleteRegions() = 0;

  /**
   * @return The current region.
   */
  virtual IRegion* currentRegion() const = 0;

  /**
   * @param aIndex Index of the region to return.
   * @return The current region.
   */
  virtual IRegion* region(int aIndex) const = 0;

  /**
   * @return The index of the current region.
   */
  virtual int currentRegionIndex() const = 0;

  /**
   * Retreive the value of a parameter knowing its key.
   * @param aKey The string key.
   * @param aValue The value.
   * @return The status of the execution.
   */
  virtual bool parameterValue(const std::string& aKey,std::string& aValue) const = 0;

  /**
   * Toggle a boolean parameter of the page.
   *
   * @param aKey The parameter to toggle.
   * @return The status of the execution. Return false if the 
   * parameter is not a boolean one.
   */
  virtual bool toggleParameter(const std::string& aKey) = 0;

  /**
   * Delete all regions except the current one.
   * Map the size of the remaining one to the size of the page.
   */
  virtual void currentRegionToPage() = 0;

  /**
   * Set the current region so that it appears in front of others.
   */
  virtual void currentRegionOnTop() = 0;

  /**
   * @return The number of managed regions.
   */
  virtual int numberOfRegions() const = 0;

  /**
   * Set the verbosity.
   * @param aVerbosity The verbosity. If zero, not verbose.
   * @return The status of execution.
   */
  virtual bool setVerboseLevel(int aVerbosity) = 0;

  /**
   * Permit to retreive the concrete implementation.
   *
   * @return The pointer of the concrete implementation object.
   * If working with Inventor, someone get a pointer to an SoPage.
   */
  virtual void* nativePage() const = 0;
};

#endif




