#ifndef OnX_IRegion_h
#define OnX_IRegion_h

/**
 * Pure abstract interface for a viewing region.
 *
 * A region is a subpart of a page that manage a 
 * scene to be visualized. A region has a specific
 * camera that can be attached to the viewer controls
 * (thumbweels).
 * 
 * @author G.Barrand
 */

#include <string>

class IRepresentation;

class IRegion {
public:
  virtual ~IRegion() {}
  virtual void* cast(const std::string&) const = 0;
public:
  /**
   * Set the size in normal coordinates (between [0,1])
   * of the region within the page.
   *
   * @param aW width in [0,1]
   * @param aH width in [0,1]
   * @return The status of the execution.
   */
  virtual bool setSize(double aW,double aH) = 0;

  /**
   * Set the position in normal coordinates (between [0,1])
   * of the region within the page. The origin is the 
   * lower left corner. The x axis points to the right
   * and the y axis points to the top.
   *
   * @param aX x position in [0,1]
   * @param aY y position in [0,1]
   * @return The status of the execution.
   */
  virtual bool setPosition(double aX,double aY) = 0;

  /**
   * Set the size in pixels of the region within the page.
   *
   * @param aW width in pixels
   * @param aH width in pixels
   * @return The status of the execution.
   */
  virtual bool setSizePixel(unsigned int aW,unsigned int aH) = 0;

  /**
   * Set the position in pixel coordinates of the region within the page. 
   * The origin is the lower left corner. The x axis points 
   * to the right and the y axis points to the top.
   *
   * @param aX x position in pixels
   * @param aY y position in pixels
   * @return The status of the execution.
   */
  virtual bool setPositionPixel(int aX,int aY) = 0;

  /**
   * Retrieve the size of the region in normal coordinate.
   *
   * @param aW width in [0,1]
   * @param aH height in [0,1]
   * @return The status of the execution.
   */
  virtual bool size(double& aW,double& aH) const = 0;

  /**
   * Retrieve the position of the region in normal coordinate.
   *
   * @param aX x position in [0,1]
   * @param aY y position in [0,1]
   * @return The status of the execution.
   */
  virtual bool position(double& aX,double& aY) const = 0;

  /**
   * Clear the scene attached to the region.
   * After this operation nothing is viewed in the region.
   *
   * @return The status of the execution.
   */
  virtual bool clear(const std::string& = "") = 0;

  /**
   * Compell an update of the scene attached to the region.
   * In general it is followed by a refresh of the window.
   * According to the implementation, some part of the scene
   * may appear, disappear or be changed.
   *
   * @return The status of the execution.
   */
  virtual bool refresh() = 0;

  /**
   * Set a parameter of the region (like background or camera position). 
   * The various parameters are very implementation specifics. 
   * If working with Inventor, the parameter is a set of words separated 
   * with colons permitting to access an SoField of the concrete
   * So[Display,Plotter,Image]Region node. 
   * For example :
   *   setParameter("color","black");  // Change the background color.
   *   setParameter("camera","height 1000 position 300 400 100");
   *   setParameter("plotter.gridStyle.visible","FALSE");
   * What can be customized are then the SoFields of the various
   * parts of an So[Display,Plotter,Image]Region nodkit. See
   * HEPVis doc (.h) of these classes for more.
   *
   * @param aParam String description of the parameter.
   * @param aValue Value to set passed by string.
   * @return The status of the execution.
   */
  virtual bool setParameter(const std::string& aParam,const std::string& aValue) = 0;

  /**
   * Get the value of a parameter of the region.
   *
   * @param aParam The parameter to get the value of.
   * @return The status of the execution.
   */
  virtual bool parameterValue(const std::string&,std::string& aParam) = 0;

  /**
   * Toggle a boolean parameter of the region.
   *
   * @param aParam The parameter to toggle.
   * @return The status of the execution. Return false if the 
   * parameter is not a boolean one.
   */
  virtual bool toggleParameter(const std::string&) = 0;

  /**
   * The entry point that permits to add new representations
   * in the viewing scene.
   *
   * @param aRepresentation The representation to add. If working with
   * Inventor, it contains the pointer to a scene graph to add 
   * in the scene.
   * @param aPlacement A string describing where to place the new
   * representation in the scene. It is implementation specific.
   * If working with a SoDisplayRegion, it could be 
   * [dynamicScene,staticScene].
   * @return The status of the execution.
   */
  virtual bool addRepresentation(IRepresentation* aRepresentation,const std::string& aPlacement = "") = 0;

  /**
   * Find a part of the scene. If working with Inventor it permits
   * to get a pointer to an SoNode in the scene graph managed
   * by a concrete So[Display,Plotter,Image]Region.
   *
   * @param aWhat The kind of information to retreive. If working with
   * Inventor it could be [name,type,part]. If "name", it permits
   * to get a named node in the scene. If "type", it permits to 
   * find the first node of a given class (SoType) in the scene.
   * If "part", it permits to get a subnode of the SoRegion nodekit itself. 
   * @param aString If aWhat is "name", the name of the SoNode to search.
   * If aWhat is "type", the class to search. If aWhat is "part", a colon
   * separated path within the SoRegion nodekit.
   * @return A pointer to n IRepresentation permitting to retreive 
   * the information or zero in case of failure.
   */
  virtual IRepresentation* findRepresentation(const std::string& aWhat,
                                              const std::string& aString) = 0;

  /**
   * Undo the last "addRepresentation". If working with Inventor
   * it permits to remove the last entered scene graph.
   */
  virtual void undo() = 0;

  /**
   * Rest the internal undo list. A further undo will then do nothing.
   */
  virtual void resetUndo() = 0;

  /**
   * Permit to share a scene graph with another region.
   * In particular it permits to view the same scene with another
   * camera positionning and zooming.
   *
   * @param aRegion The region managing the scene graph 
   * that we want to view too.
   * @return The status of the execution.
   */
  virtual bool connectFrom(const IRegion& aRegion) = 0;

  /**
   * Permit to retreive the concrete implementation.
   *
   * @return The pointer of the concrete implementation object.
   * If working with Inventor, someone get a pointer to an SoRegion.
   */
  virtual void* nativeRegion() const = 0;

  virtual std::string nativeClass() const = 0;
};

#endif




