#ifndef OnX_IUI_h
#define OnX_IUI_h

#include <Lib/Interfaces/IManager.h>

#include <vector>

namespace Lib {
  class IMessage;
}

class IWidget;
class IPage; //FIXME : deprecated
class ICyclic;
class ISession;
class ICallbackData;

typedef int (*IDispatchFunction)(void*);
typedef void *IComponent;

class IUI : public virtual IManager {
public:
  virtual ~IUI() {};
  virtual void* cast(const std::string&) const = 0;
public:
  virtual void executeScript(const std::string&,const std::string&) = 0;
  virtual void put(const std::string&) = 0;
  virtual int steer() = 0;
  virtual int steerLevel() const = 0;
  virtual void synchronize() = 0;
  virtual void exit(int = 0) = 0;
  /** 
   * If GUI in a thread, lock the thread.
   */
  virtual bool lock() = 0;

  /** 
   * If GUI in a thread, unlock the thread.
   */
  virtual bool unlock() = 0;

  virtual void echo(const std::string&) = 0;
  virtual void warn(const std::string&) = 0;
  virtual std::string ask(const std::string&) = 0;
  virtual void addDispatcher(IDispatchFunction) = 0;

  /** 
   * Set a component callback.
   * @param aName : Name of the component to modify.
   * @param aCallback : Callback to set (for example "activate" for a button).
   * @param aInterpreter : Interpreter to use for the callback.
   * @param aScript : Script to execute within the choosen interpreter when
   *                  the callback is activated.
   * @return The status of the execution.
   */
  virtual bool setCallback(const std::string& aName,
                           const std::string& aCallback,
                           const std::string& aInterpreter,
                           const std::string& aScript,
                           int aIndex = 0) = 0;
  virtual bool removeCallback(const std::string& aName,
                              const std::string& aCallback,
                              int aIndex = 0) = 0;
  virtual void show(const std::string& = "") = 0;
  virtual void hide(const std::string& = "") = 0;
  virtual bool refresh(const std::string&) = 0;
  virtual bool size(const std::string&,unsigned int&,unsigned int&) = 0;
  virtual bool position(const std::string&,int&,int&) = 0;
  virtual const std::string& callbackValue() = 0;
  virtual const ICallbackData& callbackData() = 0;
  virtual bool setParameter(const std::string&,const std::string&) = 0;
  virtual bool parameterValue(const std::string&,std::string&) = 0;
  virtual bool executeCallbacks(const std::string&,const std::string&) = 0;
  virtual bool load(const std::string&,const std::string&,bool = true) = 0;
  virtual bool save() = 0;
  virtual void deleteChildren(const std::string&) = 0;
  virtual bool write(const std::string&,
                     const std::string&,const std::string&) = 0;

  virtual IWidget* findWidget(const std::string&) = 0;

  virtual std::string currentViewer() = 0;
  virtual bool setCurrentViewer(const std::string&) = 0;

  virtual IPage* currentPage() = 0;  //FIXME : deprecated
  virtual IPage* findPage(const std::string&) = 0;  //FIXME : deprecated

  virtual ICyclic* createCyclic(const std::string&,
                                int,const std::string&,const std::string&) = 0;
  virtual ICyclic* findCyclic(const std::string&) const = 0;

  /** 
   * Create a new widget.
   * @param aName : Name of the created widget.
   * @param aClass : Class (or type) of the created widget.
   * @param aParent : Name of the parent widget.
   * @return True if success, false if failure.
   */
  virtual bool createComponent(const std::string& aName,
                               const std::string& aClass,
                               const std::string& aParent) = 0;

  /** 
   * Destroy a widget.
   * @param aName : Name of the widget to destroy.
   * @return True if success, false if failure.
   */
  virtual bool destroyComponent(const std::string& aName) = 0;

  /**
   * Copy a widget.
   * @param aName : Name of the created widget.
   * @param aComponent : Name of the widget to copy.
   * @param aParent : Name of the parent into which we put the copied widget.
   * @return True if success, false if failure.
   */
  virtual bool copyComponent(const std::string& aName,
                             const std::string& aComponent,
                             const std::string& aParent) = 0;

  /**
   * Return list of children of a component.
   * @param aComponent : Name of the component.
   * @param aWhat : What to search.
   * @param aParameter : searching parameter.
   * @return List of children.
   */
  virtual std::vector<std::string> findChildren(const std::string& aComponent,
                                                const std::string& aWhat,
                                                const std::string& aParameter) const = 0;

  /** 
   * Return list of children of a component.
   * @param aName : Name of the component.
   * @return true if component exists.
   */
  virtual bool isComponent(const std::string& aName) = 0;

  /** 
   * Return the session managing the GUI.
   *
   * @return The session.
   */
  virtual ISession& session() const = 0;

  /** 
   * @param aMessage : data sent by the emitter. 
   * When received, in the notify, someone analyse it with the "cast"
   * and have to delete it. Example :
   *      bool notify(IMessage* aMessage)  {
   *        if(KnowMessage* mess = aMessage->cast("KnownMessage")) {
   *          ...
   *        }
   *        delete aMessage;
   *      }
   *   
   * @return Status of the notification.
   */
  virtual bool notify(Lib::IMessage*) = 0;
};

#endif
